import 'package:flutter/material.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:red5pro/red5_configuration.dart';
import 'package:red5pro_example/stream_player_view.dart';
import 'package:red5pro_example/streming_view.dart';

class OptionsView extends StatefulWidget {
  const OptionsView({Key? key}) : super(key: key);

  @override
  _OptionsViewState createState() => _OptionsViewState();
}

class _OptionsViewState extends State<OptionsView> {
  GlobalKey<FormState> _formKey = GlobalKey();
  TextEditingController _licenceKeyController =
      TextEditingController(text: 'MWFI-5693-YK4B-1FDG');
  TextEditingController _hostController =
      TextEditingController(text: '178.128.27.223');
  TextEditingController _portController = TextEditingController(text: '8554');
  TextEditingController _streamNameController =
      TextEditingController(text: '810_1637654758_1');
  TextEditingController _heightController = TextEditingController(text: '720');
  TextEditingController _widthController = TextEditingController(text: '1280');
  TextEditingController _bitrateController =
      TextEditingController(text: '2500');

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Red5Pro'),
      ),
      body: Column(
        children: [
          Expanded(
            child: Form(
              key: _formKey,
              child: ListView(
                physics: ClampingScrollPhysics(),
                padding: EdgeInsets.all(10),
                children: [
                  TextFormField(
                    controller: _licenceKeyController,
                    keyboardType: TextInputType.text,
                    decoration: InputDecoration(labelText: 'License Key'),
                    validator: (value) {
                      if (value!.isEmpty) {
                        return 'Enter Licence Key';
                      }
                      return null;
                    },
                  ),
                  SizedBox(height: 10),
                  Row(
                    children: [
                      Expanded(
                        flex: 2,
                        child: TextFormField(
                          controller: _hostController,
                          keyboardType: TextInputType.url,
                          decoration: InputDecoration(labelText: 'Host'),
                          validator: (value) {
                            if (value!.isEmpty) {
                              return 'Enter Host Address';
                            }
                            return null;
                          },
                        ),
                      ),
                      SizedBox(width: 10),
                      Expanded(
                        child: TextFormField(
                          controller: _portController,
                          keyboardType: TextInputType.number,
                          decoration: InputDecoration(labelText: 'Port'),
                          validator: (value) {
                            if (value!.isEmpty) {
                              return 'Enter Port';
                            }
                            return null;
                          },
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 10),
                  TextFormField(
                    controller: _streamNameController,
                    keyboardType: TextInputType.text,
                    decoration: InputDecoration(labelText: 'Stream Name'),
                    validator: (value) {
                      if (value!.isEmpty) {
                        return 'Enter Stream Name';
                      }
                      return null;
                    },
                  ),
                  SizedBox(height: 10),
                  Row(
                    children: [
                      Expanded(
                        child: TextFormField(
                          controller: _heightController,
                          keyboardType: TextInputType.number,
                          decoration: InputDecoration(labelText: 'Height'),
                          validator: (value) {
                            if (value!.isEmpty) {
                              return 'Enter Height';
                            }
                            return null;
                          },
                        ),
                      ),
                      SizedBox(width: 10),
                      Expanded(
                        child: TextFormField(
                          controller: _widthController,
                          keyboardType: TextInputType.number,
                          decoration: InputDecoration(labelText: 'Width'),
                          validator: (value) {
                            if (value!.isEmpty) {
                              return 'Enter Width';
                            }
                            return null;
                          },
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 10),
                  TextFormField(
                    controller: _bitrateController,
                    keyboardType: TextInputType.number,
                    decoration: InputDecoration(labelText: 'Bitrate'),
                    validator: (value) {
                      if (value!.isEmpty) {
                        return 'Enter Bitrate';
                      }
                      return null;
                    },
                  ),
                ],
              ),
            ),
          ),
          Container(
            padding: EdgeInsets.all(10),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                SizedBox(
                  width: double.maxFinite,
                  child: TextButton(
                    style: TextButton.styleFrom(
                      primary: Colors.white,
                      backgroundColor: Theme.of(context).primaryColor,
                    ),
                    onPressed: () async {
                      bool _hasPermission = await _hasPermissions();
                      if (_formKey.currentState!.validate()) {
                        Navigator.of(context).push(
                          MaterialPageRoute(
                            builder: (context) => StreamingView(
                              red5configuration: _getRed5configuration(),
                              streamName: _streamNameController.text,
                            ),
                          ),
                        );
                      } else {
                        ScaffoldMessenger.of(context).showSnackBar(
                            SnackBar(content: Text('Conplete Form')));
                      }
                    },
                    child: Text('Streaming'),
                  ),
                ),
                SizedBox(
                  width: double.maxFinite,
                  child: TextButton(
                    style: TextButton.styleFrom(
                      primary: Colors.white,
                      backgroundColor: Theme.of(context).primaryColor,
                    ),
                    onPressed: () async {
                      bool _hasPermission = await _hasPermissions();
                      if (_formKey.currentState!.validate()) {
                        Navigator.of(context).push(
                          MaterialPageRoute(
                            builder: (context) => StreamPlayerView(
                              red5configuration: _getRed5configuration(),
                              streamName: _streamNameController.text,
                            ),
                          ),
                        );
                      } else {
                        ScaffoldMessenger.of(context).showSnackBar(
                            SnackBar(content: Text('Conplete Form')));
                      }
                    },
                    child: Text('Player'),
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Red5Configuration _getRed5configuration() {
    return Red5Configuration(
        licenseKey: _licenceKeyController.text,
        protocol: R5StreamProtocol.RTSP,
        host: _hostController.text,
        port: int.tryParse(_portController.text) ?? 8554);
  }

  Future<bool> _hasPermissions() async {
    Map<Permission, PermissionStatus> statuses = await [
      Permission.camera,
      Permission.microphone,
      Permission.storage,
      Permission.phone,
    ].request();
    return statuses.entries
        .where((element) =>
            element.value == PermissionStatus.denied ||
            element.value == PermissionStatus.permanentlyDenied)
        .isEmpty;
  }
}
